from events import *
from example2 import *
from network import *
import network

from twisted.spread import pb

#------------------------------------------------------------------------------
class NetworkServerView(pb.Root):
        """We SEND events to the server through this object"""
        STATE_PREPARING = 0
        STATE_CONNECTING = 1
        STATE_CONNECTED = 2

        #----------------------------------------------------------------------
        def __init__(self, evManager, sharedObjectRegistry):
                self.evManager = evManager
                self.evManager.RegisterListener( self )

                self.state = NetworkServerView.STATE_PREPARING
                self.server = None

                self.sharedObjs = sharedObjectRegistry

        #----------------------------------------------------------------------
        def Connected(self, server):
                print "CONNECTED"
                self.server = server
                self.state = NetworkServerView.STATE_CONNECTED
                ev = ServerConnectEvent( server )
                self.evManager.Notify( ev )

        #----------------------------------------------------------------------
        def ConnectFailed(self, server):
                self.state = NetworkServerView.STATE_PREPARING
                print "CONNECTION FAILED"

        #----------------------------------------------------------------------
        def Notify(self, event):
                ev = event

                if isinstance( event, TickEvent ) \
                   and self.state == NetworkServerView.STATE_PREPARING:
                        self.state = NetworkServerView.STATE_CONNECTING
                        remoteResponse = pb.getObjectAt("localhost", 8000, 30)
                        remoteResponse.addCallback(self.Connected )
                        #errback                  #self.ConnectFailed)

                #see if it's Copyable.  If not, see if there's a Copyable 
                #replacement for it.  If not, just ignore it.
                if not isinstance( event, pb.Copyable):
                        evName = event.__class__.__name__
                        if not hasattr( network, "Copyable" + evName ):
                                return
                        copyableClass = getattr( network, "Copyable" + evName )
                        ev = copyableClass( event, self.sharedObjs )

                if ev.__class__ not in clientToServerEvents:
                        #print "CLIENT NOT SENDING: " +str(ev)
                        return
                        
                #print "    ====   Client sending ===  ", str(ev)
                remoteCall = self.server.callRemote("EventOverNetwork",ev) 




#------------------------------------------------------------------------------
class NetworkServerController(pb.Referenceable):
        """We RECEIVE events from the server through this object"""
        def __init__(self, evManager, twistedReactor):
                self.evManager = evManager
                self.evManager.RegisterListener( self )

                self.reactor = twistedReactor

        #----------------------------------------------------------------------
        def remote_ServerEvent(self, event):
                #print "=================GOT AN EVENT FROM SERVER:", str(event)
                self.evManager.Notify( event )
                return 1

        #----------------------------------------------------------------------
        def remote_Ping(self):
                pong = 1
                return pong

        #----------------------------------------------------------------------
        def Notify(self, event):
                if isinstance( event, ServerConnectEvent ):
                        #tell the server that we're listening to it and
                        #it can access this object
                        remoteResponse = event.server.callRemote("ClientConnect", self)
                if isinstance( event, TickEvent ):
                        #print "PUMPING NETWORK"
                        self.reactor.iterate()


#------------------------------------------------------------------------------
class PhonyEventManager(EventManager):
        """this object is responsible for coordinating most communication
        between the Model, View, and Controller."""
        #----------------------------------------------------------------------
        def Notify( self, event ):
                pass

#------------------------------------------------------------------------------
class PhonyModel:
        """..."""
        #----------------------------------------------------------------------
        def __init__(self, evManager, sharedObjectRegistry):
                self.sharedObjs = sharedObjectRegistry
                self.game = None
                self.server = None
                self.phonyEvManager = PhonyEventManager()
                self.realEvManager = evManager
                self.neededObjects = []
                self.waitingObjectStack = []

                self.realEvManager.RegisterListener( self )

        #----------------------------------------------------------------------
         def GameReturned(self, response):
                if response[0] == 0:
                        print "GameReturned : game HASNT started"
                        #the game has not been started on the server.
                        #we'll be informed of the gameID when we receive the
                        #GameStartedEvent
                        return None
                else:
                        gameID = response[0]
                        print "GameReturned : game started ", gameID

                        self.sharedObjs[gameID] = self.game
                return self.StateReturned( response, self.GameSyncCallback )

        #----------------------------------------------------------------------
         def StateReturned(self, response):
                """this is a callback that is called in response to
                invoking GetObjectState on the server"""

                print "looking for ", response
                objID, objDict = response
                if objID == 0:
                        print "GOT ZERO -- better error handler here"
                        return None
                obj = self.sharedObjs[objID]

                success, neededObjIDs =\
                                 obj.setCopyableState(objDict, self.sharedObjs)
                if success:
                        #we successfully set the state and no further objects
                        #are needed to complete the current object
                        if objID in self.neededObjects:
                                self.neededObjects.remove(objID)

                else:
                        #to complete the current object, we need to grab the
                        #state from some more objects on the server.  The IDs
                        #for those needed objects were passed back 
                        #in neededObjIDs
                        for neededObjID in neededObjIDs:
                                if neededObjID not in self.neededObjects:
                                        self.neededObjects.append(neededObjID)
                        print "failed.  still need ", self.neededObjects
        
                self.waitingObjectStack.append( (obj, objDict) )

                retval = self.GetAllNeededObjects()
                if retval:
                        # retval is a Deferred - returning it causes a chain
                        # to be formed.  The original deferred must wait for
                        # this new one to return before it calls its next
                        # callback
                        return retval
        
        #----------------------------------------------------------------------
         def GetAllNeededObjects(self):
                if len(self.neededObjects) == 0:
                        # this is the recursion-ending condition.  If there are
                        # no more objects needed to be grabbed from the server
                        # then we can try to setCopyableState on them again and
                        # we should now have all the needed objects, ensuring
                        # that setCopyableState succeeds
                        return self.ConsumeWaitingObjectStack()

                # still in the recursion step.  Try to get the object state for
                # the objectID on the end of the stack.  Note that the 
                # recursion is done via a deferred, which may be confusing
                nextID = self.neededObjects[-1]
                print "next one to grab: ", nextID
                remoteResponse = self.server.callRemote("GetObjectState",nextID)
                remoteResponse.addCallback(self.StateReturned)
                return remoteResponse

        #----------------------------------------------------------------------
        def ConsumeWaitingObjectStack(self):
                # All the needed objects should be present now.  Just the
                # matter of setting the state on the waiting objects remains.
                while self.waitingObjectStack:
                        obj, objDict = self.waitingObjectStack.pop()
                        success, neededObjIDs =\
                                 obj.setCopyableState(objDict, self.sharedObjs)
                        if not success:
                                print "WEIRD!!!!!!!!!!!!!!!!!!"

                
        #----------------------------------------------------------------------
         def Notify(self, event):
                if isinstance( event, ServerConnectEvent ):
                        self.server = event.server
                        #when we connect to the server, we should get the
                        #entire game state.  this also applies to RE-connecting
                        if not self.game:
                                self.game = Game( self.phonyEvManager )
                        remoteResponse = self.server.callRemote("GetGame")
                        remoteResponse.addCallback(self.GameReturned)

                elif isinstance( event, CopyableGameStartedEvent ):
                        gameID = event.gameID
                        if not self.game:
                                self.game = Game( self.phonyEvManager )
                        self.sharedObjs[gameID] = self.game
                        ev = GameStartedEvent( self.game )
                        self.realEvManager.Notify( ev )

                if isinstance( event, CopyableMapBuiltEvent ):
                        mapID = event.mapID
                        if self.sharedObjs.has_key(mapID):
                                map = self.sharedObjs[mapID]
                                self.MapBuiltCallback( map )
                        else:
                                map = self.game.map
                                self.sharedObjs[mapID] = map
                                remoteResponse = self.server.callRemote("GetObjectState", mapID)
                                remoteResponse.addCallback(self.StateReturned)
                                remoteResponse.addCallback(self.MapBuiltCallback)

                if isinstance( event, CopyableCharactorPlaceEvent ):
                        charactorID = event.charactorID
                        if self.sharedObjs.has_key(charactorID):
                                charactor = self.sharedObjs[charactorID]
                                self.CharactorPlaceCallback( charactor )
                        else:
                                charactor = self.game.players[0].charactors[0]
                                self.sharedObjs[charactorID] = charactor
                                remoteResponse = self.server.callRemote("GetObjectState", charactorID)
                                remoteResponse.addCallback(self.StateReturned)
                                remoteResponse.addCallback(self.CharactorPlaceCallback)

                if isinstance( event, CopyableCharactorMoveEvent ):
                        charactorID = event.charactorID
                        if self.sharedObjs.has_key(charactorID):
                                charactor = self.sharedObjs[charactorID]
                                self.CharactorMoveCallback( charactor )
                        else:
                                charactor = self.game.players[0].charactors[0]
                                self.sharedObjs[charactorID] = charactor
                        remoteResponse = self.server.callRemote("GetObjectState", charactorID)
                        remoteResponse.addCallback(self.StateReturned)
                        remoteResponse.addCallback(self.CharactorMoveCallback)

        #----------------------------------------------------------------------
         def CharactorPlaceCallback(self, charactor):
                ev = CharactorPlaceEvent( charactor )
                self.realEvManager.Notify( ev )
        #----------------------------------------------------------------------
         def MapBuiltCallback(self, map):
                ev = MapBuiltEvent( map )
                self.realEvManager.Notify( ev )
        #----------------------------------------------------------------------
         def GameSyncCallback(self, game):
                print "sending out the GS EVENT------------------==========="
                ev = GameSyncEvent( game )
                self.realEvManager.Notify( ev )
        #----------------------------------------------------------------------
         def CharactorMoveCallback(self, charactor):
                ev = CharactorMoveEvent( charactor )
                self.realEvManager.Notify( ev )


#------------------------------------------------------------------------------
def main():
        """..."""
        evManager = EventManager()
        sharedObjectRegistry = {}

        keybd = KeyboardController( evManager )
        spinner = CPUSpinnerController( evManager )
        pygameView = PygameView( evManager )

        phonyModel = PhonyModel( evManager, sharedObjectRegistry  )

        #from twisted.spread.jelly import globalSecurity
        #globalSecurity.allowModules( network )

        from twisted.internet import reactor
        serverController = NetworkServerController( evManager, reactor )
        serverView = NetworkServerView( evManager, sharedObjectRegistry )
        
        spinner.Run()

if __name__ == "__main__":
        main()
